package com.divrep.common;

import java.io.PrintWriter;
import java.util.Random;

import org.apache.commons.lang.StringEscapeUtils;

import com.divrep.DivRep;
import com.divrep.DivRepEvent;
import com.divrep.common.DivRepFormElement;

//In order to use this location selector, you need to add following to your header
//<script src="http://maps.google.com/maps?file=api&v=2&key=_______MY_GMAP_KEY_________" type="text/javascript"></script>
public class DivRepLocationSelector extends DivRepFormElement<DivRepLocationSelector.LatLng> {
	public class LatLng
	{
		public LatLng(Double lat, Double lng, int _zoom) {
			latitude = lat;
			longitude = lng;
			zoom = _zoom;
		}
		public LatLng(Double lat, Double lng) {
			this(lat,lng, 10);
		}
		public Double latitude;
		public Double longitude;
		public int zoom;
	}

	String target_image_url;
	public DivRepLocationSelector(DivRep parent, String _target_image_url) {
		super(parent);
		target_image_url = _target_image_url;
		value = new LatLng(0.0,0.0,1);
		addClass("divrep_locationselector");
	}

	protected void onEvent(DivRepEvent e) {
		String newval = (String)e.value;
		String coords[] = newval.split(",");
		value.latitude = Double.parseDouble(coords[0]);
		value.longitude = Double.parseDouble(coords[1]);
		setFormModified();
	}

	public void render(PrintWriter out) {
		out.write("<div ");
		renderClass(out);
		out.write("id=\""+getNodeID()+"\">");
		if(!hidden) {
			if(label != null) {
				out.print("<label>"+StringEscapeUtils.escapeHtml(label)+"</label><br/>");
			}
			String random = Integer.toString(Math.abs(new Random().nextInt()));
			out.write("<div id=\"map_canvas_"+getNodeID()+random+"\" style=\"width: 500px; height: 300px\"></div>"); 
		
			out.write("<script type=\"text/javascript\">");
		    out.write("if (GBrowserIsCompatible()) {");
		    out.write("    var map = new GMap2(document.getElementById(\"map_canvas_"+getNodeID()+random+"\"));");
		    out.write("    map.setCenter(new GLatLng("+value.latitude+", "+value.longitude+"), "+value.zoom+");");
		    out.write("    map.setUIToDefault();");
		    out.write("    map.setMapType(G_HYBRID_MAP);");
		    out.write("    var prompt = new GScreenOverlay(\""+target_image_url+"\",new GScreenPoint(0.5, 0.5, 'fraction', 'fraction'), new GScreenPoint(-50, -50), new GScreenSize(100, 100, 'pixel', 'pixel'));");
		    out.write("    map.addOverlay(prompt);");
		    out.write("    GEvent.addListener(map, 'moveend', function() {divrep('"+getNodeID()+"', null, map.getCenter().toUrlValue());});");
		    out.write("}");
		    out.write("</script>");
		
			if(isRequired()) {
				out.print(lab.RequiredFieldNote());
			}
			error.render(out);	
		}
		out.write("</div>");
	}

}
